/*
 * Simple blog city -> MT (or WordPress via import) converter
 *
 * Copyright (C) 2006 Martin H. Smith
 *
 * Released under the terms of the GNU General Public License
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 */

/*
 * BCHandler.java
 *
 * Created on 26 February 2006, 15:14
 */

package com.alvermont.bcimport;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.StringTokenizer;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * <p>SAX2 ContentHandler that process the input XML file obtained from the
 * blog-city site.</p>
 *
 * <p>Note this handler does not properly deal with the document tree.
 * This was a rapidly written program for a one-off application. A normal
 * production application should make sure that the elements it is processing
 * are in the correct XML tree.</p>
 * 
 * @author martin
 * @version $Id: BCHandler.java,v 1.5 2006/02/27 10:32:53 martin Exp $
 */
public class BCHandler extends DefaultHandler implements ContentHandler
{
    /** Build up the text of the current element */
    private StringBuffer currentText = new StringBuffer();
    
    /** A list of posts we've seen */
    private List<Post> posts = new ArrayList<Post>();
    
    /** The post we're building */
    private Post currentPost;
    /** The comment we're building */
    private Comment currentComment;
    
    private DateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm");
    
    /** Creates a new instance of BCHandler */
    public BCHandler()
    {
    }

    /**
     * Try to parse a date with an attached tz offset and hope we don't
     * get caught out by all the usual issues with time zones.
     *
     * @param xmlTime A string containing the time read from the XML file
     * @return A date object representing the supplied date and time
     * @throws ParseException if there is an error parsing the string
     */
    private Date parseDate(String xmlTime) throws ParseException
    {
        String tz = xmlTime.substring(xmlTime.length() - 6);
        String dt = xmlTime.substring(0, xmlTime.length() - 6);

        long mult = 1L;
        
        if (tz.startsWith("-"))
            mult = -1L;
        
        tz = tz.substring(1);
        
        StringTokenizer tok = new StringTokenizer(tz, ":");
        
        final int hh = Integer.parseInt(tok.nextToken());
        final int mm = Integer.parseInt(tok.nextToken());
        
        final long mins = ((hh * 60) + mm) * mult;               

        final long millis = mins * 60L * 1000L;
        
        Date thedate = sdf.parse(dt);
        
        return new Date(thedate.getTime() + millis);
    }
    
    /**
     * Receive notification of character data inside an element.
     * 
     * <p>By default, do nothing.  Application writers may override this
     * method to take specific actions for each chunk of character data
     * (such as adding the data to a node or buffer, or printing it to
     * a file).</p>
     * 
     * 
     * @param ch The characters.
     * @param start The start position in the character array.
     * @param length The number of characters to use from the
     *               character array.
     * @exception org.xml.sax.SAXException Any SAX exception, possibly
     *            wrapping another exception.
     * @see org.xml.sax.ContentHandler#characters
     */
    public void characters(char[] ch, int start, int length) throws SAXException
    {
        currentText.append(ch, start, length);
    }

    /**
     * Receive notification of the end of an element.
     * 
     * <p>By default, do nothing.  Application writers may override this
     * method in a subclass to take specific actions at the end of
     * each element (such as finalising a tree node or writing
     * output to a file).</p>
     * 
     * 
     * @param uri The Namespace URI, or the empty string if the
     *        element has no Namespace URI or if Namespace
     *        processing is not being performed.
     * @param localName The local name (without prefix), or the
     *        empty string if Namespace processing is not being
     *        performed.
     * @param qName The qualified name (with prefix), or the
     *        empty string if qualified names are not available.
     * @exception org.xml.sax.SAXException Any SAX exception, possibly
     *            wrapping another exception.
     * @see org.xml.sax.ContentHandler#endElement
     */
    public void endElement(String uri, String localName, String qName) throws SAXException
    {
        try
        {
            if (localName.equals("title") && currentPost != null)
            {
                currentPost.setTitle(currentText.toString());
            }
            else if (localName.equals("body"))
            {
                if (currentComment == null)
                    currentPost.setText(currentText.toString());
                else
                    currentComment.setText(currentText.toString());
            }
            else if (localName.equals("dateupdated"))
            {
                currentPost.setPublished(parseDate(currentText.toString()));
            }
            else if (localName.equals("date"))
            {
                currentComment.setPublished(parseDate(currentText.toString()));
            }
            else if (localName.equals("user"))
            {
                currentComment.setAuthor(currentText.toString());
            }
        } 
        catch (ParseException ex)
        {
            throw new SAXException("Error parsing: " + ex.getMessage(), ex);
        }
    }

    /**
     * Receive notification of the start of an element.
     * 
     * <p>By default, do nothing.  Application writers may override this
     * method in a subclass to take specific actions at the start of
     * each element (such as allocating a new tree node or writing
     * output to a file).</p>
     * 
     * 
     * @param uri The Namespace URI, or the empty string if the
     *        element has no Namespace URI or if Namespace
     *        processing is not being performed.
     * @param localName The local name (without prefix), or the
     *        empty string if Namespace processing is not being
     *        performed.
     * @param qName The qualified name (with prefix), or the
     *        empty string if qualified names are not available.
     * @param attributes The attributes attached to the element.  If
     *        there are no attributes, it shall be an empty
     *        Attributes object.
     * @exception org.xml.sax.SAXException Any SAX exception, possibly
     *            wrapping another exception.
     * @see org.xml.sax.ContentHandler#startElement
     */
    public void startElement(String uri, String localName, String qName, 
            Attributes attributes) throws SAXException
    {
        currentText.delete(0, currentText.length());
        
        if (localName.equals("blog"))
        {
            // this is a post
            
            currentPost = new Post();
            posts.add(currentPost);
            currentComment = null;
        }
        else if (localName.equals("note"))
        {
            // this is one comment for a post
            
            currentComment = new Comment();
            currentPost.getComments().add(currentComment);
        }
    }

    /**
     * Get a (read only) list of posts from this object 
     *
     * @return A list of <code>Post</code> objects that were parsed from the file
     */
    public List<Post> getPosts()
    {
        return Collections.unmodifiableList(posts);
    }
}
